/************************************************************************
 *
 * exception.h
 *
 * (c) Copyright 2001-2008 Analog Devices, Inc.  All rights reserved.
 *
 ************************************************************************/


/*
** Definitions for user-friendly interrupt handling.
*/

/*
** Memory-Mapped Registers (MMRs) - these record what causes address
** exceptions.
*/

#define EX_DATA_FAULT_STATUS	0xFFE00008
#define EX_DATA_FAULT_ADDR	0xFFE0000C
#define EX_CODE_FAULT_STATUS	0xFFE01008
#define EX_CODE_FAULT_ADDR	0xFFE0100C

/*
** Event Vector Table
*/

#define EX_EVENT_VECTOR_TABLE	0xFFE02000

/*
** Meaning of the various bits in EXCAUSE field in SEQSTAT register.
*/

#define EX_BITS		0x3F	/* All EXCAUSE bits */
#define EX_TYPE		0x30	/* The bits which define the type */
#define EX_DEBUG	0x10	/* If set, is a debug exception type */
#define EX_SYS		0x20	/* If set, is a system exception type */
				/* If neither set, is from EXCPT instr */

#define EX_IS_DEBUG_EXCEPTION(E)	(((E)&EX_TYPE)==EX_DEBUG)
#define EX_IS_SYSTEM_EXCEPTION(E)	(((E)&EX_TYPE)==EX_SYS)
#define EX_IS_USER_EXCEPTION(E)		(((E)&EX_TYPE)==0)

/*
** Service exceptions continue from the instruction after the one
** that raised the exception.
** Error exceptions restart the instruction that raised the exception.
*/

#define EX_IS_SERVICE_EXCEPTION(E)	(!EX_IS_SYSTEM_EXCEPTION(E))
#define EX_IS_ERROR_EXCEPTION(E)	(EX_IS_SYSTEM_EXCEPTION(E))

#define EX_DB_SINGLE_STEP 0x10	/* Processor is single-stepping */
#define EX_DB_EMTRCOVRFLW 0x11	/* Emulation Trace buffer overflowed */

#define EX_SYS_UNDEFINSTR 0x21	/* Undefined instruction */
#define EX_SYS_ILLINSTRC  0x22	/* Illegal instruction combination */
#define EX_SYS_DCPLBPROT  0x23	/* Data CPLB Protection violation */
#define EX_SYS_DALIGN     0x24	/* Data access misaligned address violation */
#define EX_SYS_UNRECEVT   0x25	/* Unrecoverable event */
#define EX_SYS_DCPLBMISS  0x26	/* Data access CPLB Miss */
#define EX_SYS_DCPLBMHIT  0x27	/* Data access CPLB Multiple Hits */
#define EX_SYS_EMWATCHPT  0x28	/* Emulation watch point match */
#define EX_SYS_CACCESSEX  0x29	/* Code fetch access exception */
#define EX_SYS_CALIGN     0x2A	/* Attempted misaligned instr cache fetch */
#define EX_SYS_CCPLBPROT  0x2B	/* Code fetch CPLB Protection */
#define EX_SYS_CCPLBMISS  0x2C	/* CPLB miss on an instruction fetch */
#define EX_SYS_CCPLBMHIT  0x2D	/* Code fetch CPLB Multiple Hits */
#define EX_SYS_ILLUSESUP  0x2E	/* Illegal use of Supervisor Resource */

/*
** Meaning of the various bits in HWERRCAUSE in SEQSTAT
*/

#define EX_HWBITS (0x1F<<14)	/* bits 18:14 */

#if !defined(__ADSPLPBLACKFIN__)
#define EX_HW_NOMEM1	(0x16<<14)
#define EX_HW_NOMEM2	(0x17<<14)
#else
#define EX_HW_SYSMMR	(0x02<<14)
#define EX_HW_EXTMEM	(0x03<<14)
#endif
#define EX_HW_DMAHIT	(0x01<<14)
#define EX_HW_PERFMON (0x12<<14)
#define EX_HW_RAISE	(0x18<<14)

/*
** Meaning of the bits in DATA_FAULT_STATUS and CODE_FAULT_STATUS
*/

#define EX_DATA_FAULT_ILLADDR   (1<<19)	/* non-existent memory */
#define EX_DATA_FAULT_DAG       (1<<18)	/* 0=>DAG0, 1=>DAG1 */
#define EX_DATA_FAULT_USERSUPV  (1<<17)	/* 0=>user mode, 1=> supervisor */
#define EX_DATA_FAULT_READWRITE (1<<16)	/* 0=>read, 1=>write */
#define EX_DATA_FAULT_CPLB       0xFFFF	/* 0=>CPLB0, 1=>CPLB1, etc */

#define EX_CODE_FAULT_ILLADDR   (1<<19)	/* non-existent memory */
#define EX_CODE_FAULT_USERSUPV  (1<<17)	/* 0=>user mode, 1=> supervisor */
#define EX_CODE_FAULT_CPLB       0xFFFF	/* 0=>CPLB0, 1=>CPLB1, etc */
